-- M.A.I.D.  (Mining And Inventory Daemon)
--
-- Interface portion of M.A.I.D.
--
-- Assembled and written by igrok
-- Portions adopted from AutoJett v2.3 GUI by iry
-- Some portions taken from TGFT TradeAssistant (TA)
--

local bg_numbers = ListColors.Numbers

MAID.UI = {}


-- when timers are running, it's nice to update the UI automatically
MAID.UI.uiTimer = Timer()
MAID.UI.uiTimerDuration = 10 * 1000
MAID.UI.uiTimerFunction = {}


-- set to the name of the cargo item selected in the list
MAID.UI.cargoItemSelected = nil


MAID.UI.Show = {}        -- this will show the ui
MAID.UI.Hide = {}        -- this will hide the ui


MAID.UI.addItemPopup = {}   -- Pop-up window for adding new jettison items (see ui.addItemPopup.lua)
MAID.UI.addItemPopup.Show = {}


-- add one cargo item type to the list
function MAID:addCargoItem(name)
    -- first, make sure it isn't already in the list
    local exists = false
    for i,item in pairs(MAID.cargoItems) do
        if item.Name == name then
            exists = true
            break
        end
    end

    if not exists then
        local item = {}
        item.Name = name
        item.Jettison = false
        item.Dropped = 0
        item.Collected = 0

        table.insert(MAID.cargoItems, item)
        MAID:debugPrint('MAID added item: ' .. name)
    else
        MAID:debugPrint('MAID attempted to add duplicate item: ' .. name)
    end
end


-- remove one cargo item type from the list
function MAID:removeCargoItem(name)
    for i,item in pairs(MAID.cargoItems) do
        if item.Name == name then
            table.remove(MAID.cargoItems, i)
            MAID:debugPrint('MAID removed item: ' .. name)
            break
        end
    end
end


-- toggle the jettison of one cargo item
function MAID:toggleItemJettison(name)
    for i,item in pairs(MAID.cargoItems) do
        if item.Name == name then
            if MAID.cargoItems[i].Jettison then
                MAID.cargoItems[i].Jettison = false
            else
                MAID.cargoItems[i].Jettison = true
            end

            MAID:debugPrint('MAID toggled jettison of item: ' .. name)
            break
        end
    end
end


function MAID.UI:create_matrix(columndefs, defsort)
    local matrix
    local sortedlist
    local update_matrix
    local sort_key = defsort
    local numcolumns = (#columndefs)
    local function set_sort_mode(mode)
        -- clicked on title of column
        sort_key = mode
        -- color the text accordingly
        for i=1,numcolumns do
            matrix:setattribute("BGCOLOR", 0, i, mode == i and tabseltextcolor or tabunseltextcolor)
        end
    end

    matrix = iup.pdasubsubmatrix{
        numcol = numcolumns,
        expand = "YES",
        size="200x100",
    }
    for i=1,numcolumns do
        matrix["ALIGNMENT"..i] = columndefs[i].alignment
        matrix["0:"..i] = columndefs[i].title
    end
    set_sort_mode(defsort)
    function matrix:bgcolor_cb(row, col)
        local c = bg_numbers[math.fmod(row,2)]
        return c[1],c[2],c[3],c[4],iup.DEFAULT
    end
    --matrix.fgcolor_cb = matrix.bgcolor_cb
    function matrix:click_cb(row, col)
        if row == 0 and tonumber(matrix.NUMLIN) > 0 then
            set_sort_mode(col)
            update_matrix(self)
        elseif columndefs.on_sel and sortedlist then
            columndefs.on_sel(matrix, row, sortedlist[row])
            update_matrix(self)
        end
    end
    function matrix:edition_cb()
        return iup.IGNORE
    end

    local function sort_list(sort_key, list)
        table.sort(list, columndefs[sort_key].fn)
    end

    local function reload_matrix(matrix, list)
        matrix.numlin = (#list)
    
        for index,v in ipairs(list) do
            columndefs.update_entry(matrix, index, v)
        end
    end
    
    local function reset_matrix(self, characterlist)
        matrix.dellin = "1--1"  -- one way of deleting all items in the matrix
        sortedlist = characterlist
        sort_list(sort_key, sortedlist)
        reload_matrix(matrix, sortedlist)
    end

    update_matrix = function(self)
        set_sort_mode(sort_key)
            sort_list(sort_key, sortedlist)
            reload_matrix(matrix, sortedlist)
    end

    matrix.update = update_matrix
    matrix.reset = reset_matrix

    return matrix
end


-- iup objects for the settings tab 
MAID.UI.jettisonToggle = iup.stationtoggle{title = "Jettison Active", value = MAID:StrBool(MAID.settings.jettison), action = function() MAID:Toggle() end}
MAID.UI.exclusiveModeToggle = iup.stationtoggle{title = "Exclusive Mode", value = MAID:StrBool(MAID.settings.exclusiveMode), action = function() MAID:ToggleExclusiveMode() end}
MAID.UI.combatModeToggle = iup.stationtoggle{title = "Combat Mode", value = MAID:StrBool(MAID.settings.combatMode), action = function() MAID:ToggleCombatMode() end}
MAID.UI.tempNotifyToggle = iup.stationtoggle{title = "Notify", value = MAID:StrBool(MAID.settings.tempNotify), action = function() MAID:ToggleTempNotify() end}
MAID.UI.tempSafetyToggle = iup.stationtoggle{title = "Safety", value = MAID:StrBool(MAID.settings.tempSafety), action = function() MAID:ToggleTempSafety() end}
MAID.UI.temperatureTextBox = iup.text{expand="HORIZONTAL", action=function(self, key, value)
        if key == 13 and value ~= "" then MAID.UI.setTemperatureButton:action() end
    end}
MAID.UI.setTemperatureButton = iup.stationbutton{title="Set", action = function()
        local temperature = MAID.UI.temperatureTextBox.value
        MAID:SetSafeTemperature(temperature)
        MAID.UI.reset_data()
    end}
MAID.UI.soundToggle = iup.stationtoggle{title = "Sound", value = MAID:StrBool(MAID.settings.sound), action = function() MAID:ToggleSound() end}

MAID.UI.settingsBox = iup.vbox{
    iup.pdarootframe{
        iup.pdasubframebg{
                iup.hbox{
                    iup.vbox{
                        MAID.UI.jettisonToggle,
                        iup.fill{size="15"},
                        MAID.UI.exclusiveModeToggle,
                        iup.fill{size="15"},
                        MAID.UI.combatModeToggle,
                    },
                    iup.fill{size="30"},
                    iup.vbox{
                        iup.label{title="Automatically jettison unwanted cargo from the ship."},
                        iup.fill{size="15"},
                        iup.label{title="Jettison any items not on the cargo jettison list."},
                        iup.fill{size="15"},
                        iup.label{title="Immediately jettison *all* cargo from the ship."},
                    },
                    iup.fill{expand="HORIZONTAL"},
                    margin="5x5",
                },
        },
    },
    iup.fill{size="30"},

    iup.pdarootframe{
        iup.pdasubframebg{
            iup.vbox{
                iup.hbox{
                    iup.vbox{
                        MAID.UI.tempNotifyToggle,
                        iup.fill{size="15"},
                        MAID.UI.tempSafetyToggle,
                        iup.fill{size="15"},
                        iup.label{title=" "},
                        iup.label{title=" "},
                        MAID.UI.soundToggle,
                    },
                    iup.fill{size="30"},
                    iup.vbox{
                        iup.label{title="Notify when asteroid has reached too high a temperature."},
                        iup.fill{size="15"},
                        iup.label{title="Power down mining beams when asteroid temperature is too high."},
                        iup.label{title="Mining beams should be assigned to one weapon group, and"},
                        iup.label{title="the same beam should not be bound to multiple keys/buttons."},
                        iup.fill{size="15"},
                        iup.label{title="Play a sound to notify of asteroid overheating."},
                    },
                    iup.fill{expand="HORIZONTAL"},
                },
                iup.fill{size="30"},

                iup.hbox{
                    iup.label{title="Maximum asteroid temperature: "},
                    MAID.UI.temperatureTextBox,
                    iup.fill{size="10"},
                    MAID.UI.setTemperatureButton,
                },
                margin="5x5",
            },
        },
    },
}

local settings_tab = iup.vbox{
    MAID.UI.settingsBox,
    tabtitle = 'Settings',
}
--- end settings tab


-- text colors for cargo items
MAID.UI.itemColors = { -- color codes , use rgbtohex(value) to get \127RRGGBB in hex
    commodities     = '40 180 240',
    ship            = '126 255 128',
    lightweapon     = '127 127 127',
    heavyweapon     = '153 153 153',
    battery         = '255 185 53',
    mining          = '100 150 0',
    turretweapon    = '200 150 0',
    unknown         = '255 255 255',
}
-- text colors for the ores
MAID.UI.orecolors = {
    ['Premium Aquean Ore'] = 'aec6d6',
    ['Premium Silicate Ore'] = '9c8a5f',
    ['Premium Carbonic Ore'] = '749281',
    ['Premium Ferric Ore'] = 'a4a39b',
    ['Premium Ishik Ore'] = '12d7e7',
    ['Premium VanAzek Ore'] = 'dda994',
    ['Premium Xithricite Ore'] = '16ef25',
    ['Lanthanic Ore'] = 'a320e5',
    ['Denic Ore'] = '13a2d9',
    ['Pyronic Ore'] = 'e38163',
    ['Apicene Ore'] = '25578d',
    ['Pentric Ore'] = 'f9f825',
    ['Heliocene Ore'] = 'd73d25',
}

local function colorizeItem(itemname)
    -- color for unknown items
    local itemcolor = ' \127ffffff'

    if MAID.UI.orecolors[itemname] then
        itemcolor = ' \127' .. MAID.UI.orecolors[itemname]
    --elseif itemname:match("Mining") or itemname:match("Mineral Scanner") then
    --    itemcolor = rgbtohex(MAID.UI.itemColors.mining)
    --elseif not MAID.UI.itemColors[v.type] then
    --    itemcolor = rgbtohex(MAID.UI.itemColors.unknown)
    --else
    --    itemcolor = rgbtohex(MAID.UI.itemColors[v.type])
    end

    return itemcolor
end


-- iup objects for the cargo items tab 
local cargoitems_sorttbl = {
    [1] = {
        title = "Name",
        alignment = "ALEFT",
        fn = function(a,b)
            -- place mined ores at the top of the list
            if a.Name:match(' Ore') and b.Name:match(' Ore') then
                -- place the 'premium' ores above the others
                if a.Name:match('Premium ') and b.Name:match('Premium ') then
                    return a.Name < b.Name
                elseif a.Name:match('Premium ') then
                    return true
                elseif b.Name:match('Premium ') then
                    return false
                else
                    return a.Name < b.Name
                end
            elseif a.Name:match(' Ore') then
                return true
            elseif b.Name:match(' Ore') then
                return false
            else
                return a.Name < b.Name
            end
        end,
        },
    [2] = {
        title = "Jettison",
        alignment = "ACENTER",
        fn = function(a,b)
            if a.Jettison == b.Jettison then
                return a.Name < b.Name
            else
                return MAID:StrBool(a.Jettison) > MAID:StrBool(b.Jettison)
            end
        end,
        },
    [3] = {
        title = "Dropped",
        alignment = "ARIGHT",
        fn = function(a,b)
            if a.Dropped == b.Dropped then
                return a.Name < b.Name
            else
                return a.Dropped > b.Dropped
            end
        end,
        },
    [4] = {
        title = "Collected",
        alignment = "ARIGHT",
        fn = function(a,b)
            if a.Collected == b.Collected then
                return a.Name < b.Name
            else
                return a.Collected > b.Collected
            end
        end,
        },
    [5] = {
        title = "Expiration",
        alignment = "ARIGHT",
        fn = function(a,b)
            return a.Name < b.Name
        end,
        },
    update_entry = function(matrix, index, data)
        local dullColor = ' \127666666'

        matrix:setcell(index, 1, colorizeItem(data.Name) .. data.Name)

        if data.Jettison then
            matrix:setcell(index, 2, '*')
        else
            matrix:setcell(index, 2, ' ')
        end

        if data.Dropped > 0 then
            matrix:setcell(index, 3, ' \127aa0000' .. tostring(data.Dropped))
        else
            matrix:setcell(index, 3, dullColor .. tostring(data.Dropped))
        end

        if data.Collected > 0 then
            matrix:setcell(index, 4, ' \12700aa00' .. tostring(data.Collected))
        else
            matrix:setcell(index, 4, dullColor .. tostring(data.Collected))
        end

        matrix:setcell(index, 5, data.Timer)
    end,
    on_sel = function (matrix, row, data)
        if(data ~= nil) then
            MAID.UI.cargoItemSelected = data.Name
        end
    end,
}

MAID.UI.cargoitems_matrix = MAID.UI:create_matrix(cargoitems_sorttbl, 1)
MAID.UI.addButton = iup.stationbutton{title="Add Item...", expand="HORIZONTAL", hotkey=iup.K_a, action = function() MAID.UI.addItemPopup:Show() end}
MAID.UI.toggleButton = iup.stationbutton{title="Toggle Jettison", expand="HORIZONTAL", hotkey=iup.K_t, action = function()
        MAID:toggleItemJettison(MAID.UI.cargoItemSelected)
        MAID.UI:reset_data()
    end}
MAID.UI.removeButton = iup.stationbutton{title="Remove Selected Item", hotkey=iup.K_r, action = function()
        MAID:removeCargoItem(MAID.UI.cargoItemSelected)
        MAID.UI:reset_data()
    end}

local cargoitems_tab = iup.vbox{
    iup.pdarootframe{
        iup.pdasubframebg{
            iup.vbox{
                iup.label{title="Select the cargo items to track:"},
                MAID.UI.cargoitems_matrix,
                iup.hbox{
                    MAID.UI.addButton,
                    MAID.UI.toggleButton,
                    MAID.UI.removeButton,
                },
            },
        },
    },
    tabtitle = 'Cargo Jettison List',
}
--- end cargo items tab


MAID.UI.MAIN = iup.dialog{
    iup.stationhighopacityframe{
        iup.stationhighopacityframebg{
            iup.vbox{
                iup.pda_root_tabs{
                    settings_tab,
                    cargoitems_tab,
                },
            },
        },
    },
    border="YES",
    topmost="YES",
    size='%80x%60',
    DEFAULTESC=MAID.UI.Hide,
    BGCOLOR = "0 0 0 256 *",
    title = 'M.A.I.D. v' .. MAID.version,
}
MAID.UI.MAIN:map()


function MAID.UI:reset_data()
    MAID.UI.jettisonToggle.value = MAID:StrBool(MAID.settings.jettison)
    MAID.UI.exclusiveModeToggle.value = MAID:StrBool(MAID.settings.exclusiveMode)
    MAID.UI.combatModeToggle.value = MAID:StrBool(MAID.settings.combatMode)
    MAID.UI.tempNotifyToggle.value = MAID:StrBool(MAID.settings.tempNotify)
    MAID.UI.tempSafetyToggle.value = MAID:StrBool(MAID.settings.tempSafety)
    MAID.UI.soundToggle.value = MAID:StrBool(MAID.settings.sound)
    MAID.UI.temperatureTextBox.value = MAID.settings.tempMax

    -- fill the cargo matrix with the relevant data
    MAID.UI.cargoitems_tbl = {}
    for i,item in pairs(MAID.cargoItems) do
        -- check timer
        local timer = MAID.time:remaining(item.Name)
        if timer == nil then
            timer = " "
        else
            timer = MAID.time:msec2str(timer)
        end

        table.insert(MAID.UI.cargoitems_tbl,1,{
                Name = item.Name,
                Jettison = item.Jettison,
                Dropped = item.Dropped,
                Collected = item.Collected,
                Timer = timer,
            }
        )
    end
    MAID.UI.cargoitems_matrix:reset(MAID.UI.cargoitems_tbl)

    -- width of the matrix columns
    local width = getwidth(MAID.UI.cargoitems_matrix) - Font.Default
    MAID.UI.cargoitems_matrix.width1 = width/7 * 3     -- Name
    MAID.UI.cargoitems_matrix.width2 = width/7         -- Jettison
    MAID.UI.cargoitems_matrix.width3 = width/7         -- Dropped
    MAID.UI.cargoitems_matrix.width4 = width/7         -- Collected
    MAID.UI.cargoitems_matrix.width5 = width/7         -- Timer
end


function MAID.UI.uiTimerFunction()
    MAID.UI:reset_data()
    MAID.UI.uiTimer:SetTimeout(MAID.UI.uiTimerDuration, MAID.UI.uiTimerFunction)
end


function MAID.UI:Show()
    MAID.UI:reset_data()
    ShowDialog(MAID.UI.MAIN,iup.CENTER,iup.CENTER)

    MAID.UI.uiTimer:SetTimeout(MAID.UI.uiTimerDuration, MAID.UI.uiTimerFunction)
end

function MAID.UI:Hide()
    HideDialog(MAID.UI.MAIN,iup.CENTER,iup.CENTER)

    MAID.UI.uiTimer:Kill()
end



-- this cute snipet allows the app to close/hide when the bound key is pressed
-- i.e. bind 'x' to /storm press x once it launches /storm, press again it closes
function MAID.UI.MAIN:k_any(char)
    local command = gkinterface.GetCommandForKeyboardBind(char)
    if command == 'maid' then
        MAID.UI.Hide()
        return iup.IGNORE
    end
    return iup.CONTINUE
end

